/////////////////////////////////////////////////////////////////////////////
// ListBoxControlX.cpp : Implementation of CListBoxControlX
//
// Ownerdrawn Superclassed ListBox ActiveX Control
//
// All rights reserved.
//
// Written by Naveen Kohli (naveen@a3ds.com)
// Version 1.0
//
// Distribute freely, except: don't remove my name from the source or
// documentation (don't take credit for my work), mark your changes (don't
// get me blamed for your possible bugs), don't alter or remove this
// notice.
// No warrantee of any kind, express or implied, is included with this
// software; use at your own risk, responsibility for damages (if any) to
// anyone resulting from the use of this software rests entirely with the
// user.
//
// Send bug reports, bug fixes, enhancements, requests, flames, etc. to
//    naveen@a3ds.com
/////////////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "ATLListControl.h"
#include "ListBoxControlX.h"
#include "ocidl.h"	// Added by ClassView

#define BUFFER MAX_PATH

//
//	Function:
//		CListBoxControlX()
//
//	Parameters:
//
//	Retruns:
//	
//	Remarks:
//		Function is default constructor for the CListBoxControlX class. Since we are
//		superclassing a ListBox control, there the Window class name has been set to
//		ListBox in the parameter intitialization. And since ListBox control can only be
//		created with a Window, therefore m_bwindowOnly parameter for the control has
//		been set to TRUE.
//

CListBoxControlX::CListBoxControlX () :	
		m_ctlListBox (_T("ListBox"), this, 1)
{
	m_bWindowOnly = TRUE;
	m_clrDividerLineColor = RGB (255, 0, 0);

	// Create the default font object.

	static FONTDESC fontDesc = {sizeof (FONTDESC), OLESTR ("MS Sans Serif"), FONTSIZE (12),
		FW_BOLD, ANSI_CHARSET, FALSE, FALSE, FALSE};

	OleCreateFontIndirect (&fontDesc, IID_IFontDisp, (void **)&m_pFont);
}

//
//	Function:
//		STDMETHODIMP get_DividerLineColor(OLE_COLOR *pVal)
//
//	Parameters:
//		pVal		Pointer to new color value
//
//	Returns:
//		HRESULT value
//
//	Remarks:
//		Call this function to get the color of the divider line.
//

STDMETHODIMP
CListBoxControlX::get_DividerLineColor(OLE_COLOR *pVal)
{
	if (NULL == pVal) {
		return Error (OLESTR ("NULL pointer specified for return value"),
				IID_IListBoxControlX, E_POINTER);
	}

	*pVal = m_clrDividerLineColor;

	return S_OK;
}

//
//	Function:
//		STDMETHODIMP put_DividerLineColor(OLE_COLOR newVal)
//
//	Parameters:
//		newVal		Value of new color
//
//	Returns:
//		HRESULT value
//
//	Remarks:
//		Call this function to set a new color for the divider line. This function not
//		only sets a new value bit also informas the containers and all connected objects
//		to control about the change in color. This has been done cause this property
//		has been marked with bindable and requestedit attributes in the IDL file.
//

STDMETHODIMP
CListBoxControlX::put_DividerLineColor(OLE_COLOR newVal)
{
	// If the color is same as that previously set, there is no need to change
	// anthing.

	if (m_clrDividerLineColor == newVal) {
		return S_OK;
	}

	// Make sure that container has not frozen event notifications. If not, then fire
	// OnRequestEdit event to ask the container for permission to change the Divider Line
	// color property.

	if (!m_nFreezeEvents) {
		// Notify all connected interfaces that Divider Line color is about to be
		// changed. This method can only be called if the control class is derived
		// from IPropertyNotifySink. Otherwise this method will return S_OK.

		if (FireOnRequestEdit (DISPID_DIVIDERLINECOLOR) == S_FALSE) {
			return Error (OLESTR ("Failed to notify connected containers about line color change"),
				IID_IListBoxControlX, E_FAIL);
		}
	}

	// Save the new color value.

	m_clrDividerLineColor = newVal;

	// Set the dirry falg for control persistence cause we have changed control property.

	m_bRequiresSave = TRUE;

	// Make sure that container has not frozen event notifications.

	if (!m_nFreezeEvents) {
		// Notify all connected IPropertyNotifySink interfaces that the Divider Line color
		// property has changed. If the control class does not derive from IPropertyNotifySink,
		// this method returns S_OK. 

		FireOnChanged (DISPID_DIVIDERLINECOLOR);
	}

	// Update the control view.

	FireViewChange ();

	// Notifies all advisory sinks registered with the advise holder that the control data
	// has changed. 

	SendOnDataChange (NULL);

	return S_OK;
}

//
//	Function:
//		BOOL PreTranslateAccelerator(LPMSG pMsg, HRESULT& hRet)
//
//	Parameters:
//		pMsg	Pointer to message
//		hRet	Result value
//
//	Retruns:
//		BOOL value.
//
//	Remarks:
//		Function gets called by container to check for accelerator keys handled by control.
//

BOOL
CListBoxControlX::PreTranslateAccelerator(LPMSG pMsg, HRESULT& hRet)
{
	if(pMsg->message == WM_KEYDOWN && 
		(pMsg->wParam == VK_LEFT || 
		pMsg->wParam == VK_RIGHT ||
		pMsg->wParam == VK_UP ||
		pMsg->wParam == VK_DOWN))
	{
		hRet = S_FALSE;
		return TRUE;
	}
	//TODO: Add your additional accelerator handling code here
	return FALSE;
}

//
//	Function:
//		LRESULT OnSetFocus (UINT uMsg, WPARAM wParam, LPARAM lParam, BOOL& bHandled)
//	Parameters:
//		uMsg		Message code
//		wParam		WPARAM value for message
//		lParam		LPARAM value for the message
//		bHandled	TRUE if message handled by application
//
//	Retruns:
//		Result value
//
//	Remarks:
//		Function gets called when the ListBox control gets focus. First it cheks that the
//		control is in place active or not. This means the control is visible and its window,
//		if any, is visible, but its menus and toolbars may not be active. Then it activates
//		the control's user interface with DoVerbUIActivate call. Then it verfies that control
//		does not have focus and give it focus.
//

LRESULT
CListBoxControlX::OnSetFocus (UINT uMsg, WPARAM wParam, LPARAM lParam, BOOL& bHandled)
{
	LRESULT lRes = CComControl<CListBoxControlX>::OnSetFocus(uMsg, wParam, lParam, bHandled);
	
	// Make sure that control is InPlaceActive
	if (m_bInPlaceActive)
	{
		DoVerbUIActivate(&m_rcPos,  NULL);
		if(!IsChild(::GetFocus()))
			m_ctlListBox.SetFocus();
	}
	return lRes;
}

//
//	Function:
//		LRESULT OnCreate (UINT uMsg, WPARAM wParam, LPARAM lParam, BOOL& bHandled)
//
//	Parameters:
//		uMsg		Message code
//		wParam		WPARAM value for message
//		lParam		LPARAM value for the message
//		bHandled	TRUE if message handled by application
//
//	Retruns:
//		LRESULT value
//
//	Remarks:
//		This function gets called when control recieves WM_CREATE message that when it is
//		to created first time. You can specify the window styles to be specified for the
//		creation. Since we are creating an owner drawn control therefore LBS_OWNERDRAWFIXED
//		style has been added. For the list control to have strings only, LBS_HASSTRINGS
//		style has been specified. And this control will be notifying the actions taken
//		on the strings in list, LBS_NOTIFY flag has to added to style.
//

LRESULT
CListBoxControlX::OnCreate (UINT uMsg, WPARAM wParam, LPARAM lParam, BOOL& bHandled)
{
	DWORD dwStyle = (WS_CHILD | WS_VISIBLE | WS_HSCROLL | WS_VSCROLL | WS_BORDER |
		LBS_DISABLENOSCROLL | LBS_OWNERDRAWFIXED | LBS_NOTIFY | LBS_HASSTRINGS);

	RECT rc;
	GetWindowRect(&rc);
	rc.right -= rc.left;
	rc.bottom -= rc.top;
	rc.top = rc.left = 0;
	m_ctlListBox.Create(m_hWnd, rc, NULL, dwStyle);
	return 0;
}

//
//	Function:
//		HRESULT FinalContruct ()
//
//	Parameters:
//
//	Returns:
//		HRESULT value
//
//	Remarks:
//		This function gets called before the control's construction is complete. The
//		put and get methods of Stock properties fire OnRequestEdit and OnChanged events.
//		The control should hold off the firing of these events before the control gets
//		fully constructed. Some of the containers break if this rule is not is not obeyed.
//		Therefore FreezeEvenets (TRUE) disables the event firing. And before we exit from
//		this function, FreezeEvenets (FALSE) gets called to reenable the event firing.
//

HRESULT
CListBoxControlX::FinalConstruct ()
{
    FreezeEvents (TRUE);

	// TODO: Put any control initialization code here.

    FreezeEvents (FALSE);
    return S_OK;
}

//
//	Function:
//		OnDraw (ATL_DRAWINFO &di)
//
//	Parameters:
//		di		Reference to ATL_DRAWINFO data structure
//
//	Retruns:
//		HRESULT type
//
//	Remarks:
//		Since we have superclasses ListBox control and its ownerdrawn, therefore we need
//		not handle any drawing code in this function.
//

HRESULT
CListBoxControlX::OnDraw (ATL_DRAWINFO& di)
{

	return S_OK;
}

//
//	Function:
//		LRESULT OnDrawItem (UINT uMsg, WPARAM wParam, LPARAM lParam, BOOL& bHandled)
//
//	Parameters:
//		uMsg		Message code
//		wParam		WPARAM value for message
//		lParam		LPARAM value for the message
//		bHandled	TRUE if message handled by application
//
//	Retruns:
//		LRESULT value
//
//	Remarks:
//		This ListBox is ownerdrawn type, therefore all the control drawing has to be handled
//		when control gets WM_DRAWITEM message from parent window. The LPARAM value corresponds
//		to DRAWITEMSTRUCT which contains all the information about the string item in list
//		box control for which drawing has to take place. Function checks if the current item
//		corresponds to one of the divider indices, then it draws a line. Otherwise the text
//		string is drawn on device Context. One thing which has to be taken care of in this
//		function is that The application should restore all graphics device interface (GDI)
//		objects selected for the display context supplied in LPDRAWITEMSTRUCT before this member
//		function terminates.


LRESULT
CListBoxControlX::OnDrawItem (UINT uMsg, WPARAM wParam, LPARAM lParam, BOOL& bHandled)
{
	int selId;
	POINT topLeft, rightBottom;
	LPDRAWITEMSTRUCT lpDrawItem;
	HBRUSH bkBrush;
	HPEN hNewPen, hOldPen;
	
	lpDrawItem = (LPDRAWITEMSTRUCT)lParam;
	HDC hDC = lpDrawItem->hDC;
	RECT rc = lpDrawItem->rcItem;

	topLeft.x = rc.left;
	topLeft.y = rc.top;
	rightBottom.x = rc.right;
	rightBottom.y = rc.bottom;

	// At design time there will not be any items in the list, therefore we don't need
	// to handle drawing operations. So check if the itemID is non-zero or not.

	if ((selId = lpDrawItem->itemID) < 0) {
		return 0;
	}

	// If the itemID corresponds to one of the divider indices, draw the divider line
	// with the color specified by user. If no color is specified then the default color
	// i.e. RED is used.

	if (FindDividerIndex (selId) != -1) {
		int halfHeight = abs ((rc.bottom - rc.top) / 2.0);
		int halfWidth = abs (rc.right - rc.left);

		// We could have added another property type for user to specify line width to
		// be used for divider line.

		int lineStyle = PS_SOLID;

		// Create a pen for drawing divider line.

		hNewPen = (HPEN)::CreatePen (lineStyle, 2, m_clrDividerLineColor);
		
		// Create solid brush with white background.

		bkBrush = ::CreateSolidBrush (RGB (255, 255, 255));

		// Create solid brush with white background.

		SetBkMode (hDC, TRANSPARENT);

		// Select the created pen in DC.

		hOldPen = (HPEN)::SelectObject (hDC, hNewPen);

		// Draw the divider line.

		::MoveToEx (hDC, rc.left, rc.top + halfHeight, NULL);
		::LineTo (hDC, rc.right, rc.top + halfHeight);
		
		// Restore the old pen in DC.

		::SelectObject (hDC, hOldPen);
	}
	else {
		if (lpDrawItem->itemState & (ODS_FOCUS | ODS_SELECTED)) {
			// If the item in focus is being drawn then set the texy color to white and
			// background color to highlight color.

			SetTextColor (hDC, RGB (255, 255, 255));
			bkBrush = CreateSolidBrush (COLOR_HIGHLIGHT);
			SetROP2 (hDC, R2_NOT);
		}
		else {
			// For normal item drawing, set the text color to black and background to
			// white.

			SetTextColor (hDC, RGB (0, 0, 0));
			bkBrush = CreateSolidBrush (RGB (255, 255, 255));
		}

		FillRect (hDC, &rc, bkBrush);
		SetBkMode (hDC, TRANSPARENT);
		
		TCHAR tchBuffer [BUFFER];
		if (::SendMessage (lpDrawItem->hwndItem, LB_GETTEXT, selId,
			(LPARAM) tchBuffer) == LB_ERR) {
			_ASSERTE (false);
		}

		TextOut (hDC, topLeft.x, topLeft.y, tchBuffer, strlen (tchBuffer));
	}

	return 0;
}

//
//	Function:
//		LRESULT OnSelectionChange (UINT uMsg, WPARAM wParam, LPARAM lParam, BOOL& bHandled)
//
//	Parameters:
//		uMsg		Message code
//		wParam		WPARAM value for message
//		lParam		LPARAM value for the message
//		bHandled	TRUE if message handled by application
//
//	Retruns:
//		LRESULT value
//
//	Remarks:
//		This function handles the WM_COMMAND messages sent to the control. When the selection
//		in the list box is about to change, ListBox sends WM_COMMAND message to the parent
//		window. This function gives a chance to do special handling and then decide if the
//		control will handle the message by itself or let the default message handlers handle
//		it. If the notification code is LBN_SELCHANGE, then another check is done to see if
//		user has clicked on one of the divider lines. If so then the default message handling
//		is allowed to take place. Otherwise SelectionChange is fired for the control container.
//

LRESULT
CListBoxControlX::OnSelectionChange (UINT uMsg, WPARAM wParam, LPARAM lParam, BOOL& bHandled)
{
	long curSel;
	DWORD wNotifyCode, wID;
	HWND hwndCtl = NULL;

	// Get the notification message and the handle to control window.

	wNotifyCode = HIWORD (wParam);
	wID = LOWORD (wParam);
	hwndCtl = HWND (lParam);

	if (wNotifyCode == LBN_SELCHANGE && !FAILED (GetCurSel (&curSel)) &&
		hwndCtl == m_ctlListBox.m_hWnd) {
		if (FindDividerIndex (curSel) == -1) {
			Fire_SelectionChange ();
			return 0;
		}
	}

	m_ctlListBox.DefWindowProc(uMsg, wParam, lParam);
	return 0;
}

//
//	Function:
//		STDMETHODIMP SetObjectRects (LPCRECT prcPos,LPCRECT prcClip)
//
//	Parameters:
//		prcPos		Control's rectangle
//		prcClip		Clipping rectangle
//
//	Retruns:
//		HRESULT value
//
//	Remarks:
//		Called by the container to inform the control that its size and/or position has
//		changed. Then the control can set the appropriate window position.
//

STDMETHODIMP
CListBoxControlX::SetObjectRects (LPCRECT prcPos,LPCRECT prcClip)
{
	IOleInPlaceObjectWindowlessImpl<CListBoxControlX>::SetObjectRects(prcPos, prcClip);
	int cx, cy;
	cx = prcPos->right - prcPos->left;
	cy = prcPos->bottom - prcPos->top;
	::SetWindowPos(m_ctlListBox.m_hWnd, NULL, 0,
		0, cx, cy, SWP_NOZORDER | SWP_NOACTIVATE);
	return S_OK;
}

//
//	Function:
//		GetInterfaceSafetyOptions (REFIID riid, DWORD *pdwSupportedOptions,
//			DWORD *pdwEnabledOptions)
//
//	Parameters:
//		riid				Specifies the interface ID for a given object.
//		pdwSupportedOptions Specifies the options supported for the interface identified by riid.
//		pdwEnabledOptions	Specifies the options currently enabled for the interface identified
//							by riid. 
//
//	Return Values:
//		Returns S_OK if successful, or E_NOINTERFACE if riid specifies an interface that is unknown
//		to the object
//
//	Comments:
//		This method returns a set of bits in pdwSupportedOptions for each capability that the
//		control knows about, and a set of bits in pdwEnabledOptions for each capability for which
//		the control is currently safe. 
//
//	NOTE: There is a mistake in Microsoft's documentation on IObjectSafetyImpl interface. The
//		  member variable which gives the safety option for interface is m_dwCurrentSafety. But in
//		  the Knowledgebase articales and the online documentation it has been written as
//		  m_dwSafety.


STDMETHODIMP
CListBoxControlX::GetInterfaceSafetyOptions (REFIID riid, DWORD *pdwSupportedOptions,
		DWORD *pdwEnabledOptions)
{
	HRESULT hRes = E_FAIL;
	LPUNKNOWN pUnknown;

	// Check if the interface even support this interface.

	hRes = _InternalQueryInterface (riid, (void **) &pUnknown);
	if (FAILED (hRes)) {
		ATLTRACE (_T ("Failed to get interface pointer!\n"));
		return Error (OLESTR ("Interface not supported by control"),
			IID_IListBoxControlX, E_NOTIMPL);
	}

	// Make sure that the passed argument pointers are valid.

	if (pdwSupportedOptions == NULL || pdwEnabledOptions == NULL) {
		ATLTRACE (_T ("Null pointers in the arguments!\n"));
		return Error (OLESTR ("Invalid safety type arguments"),
			IID_IListBoxControlX, E_INVALIDARG);
	}

	// Release the inteface pointer.

	pUnknown->Release ();
	pUnknown = NULL;

	// Check what category the iid belongs to.

	if (riid == IID_IDispatch) {
		// IDispatch is interface used for scripting. If the control supports IDispatch or Dual
		// interfaces, then mark the control as safe for scripting. Make sure that you don't play
		// with this option because user of your control relies on you.
		
		*pdwSupportedOptions = INTERFACESAFE_FOR_UNTRUSTED_CALLER;
		*pdwEnabledOptions = m_dwCurrentSafety & INTERFACESAFE_FOR_UNTRUSTED_CALLER;

		return (S_OK);
	}
	else if (riid == IID_IPersistStreamInit || riid == IID_IPersistStorage ||
		riid == IID_IPersistPropertyBag) {
		// The persistence interfaces are used for intialization. If other persistence interfaces
		// are supported also, add in this section. Indicate to user that control is safe for
		// initializing.

		*pdwSupportedOptions = INTERFACESAFE_FOR_UNTRUSTED_DATA;
		*pdwEnabledOptions = m_dwCurrentSafety & INTERFACESAFE_FOR_UNTRUSTED_DATA;

		return (S_OK);
	}
	else {
		// If iid belongs to none of the supported persistence or dispatch interface then
		// mark the control unsafe for initializing or scripting.

		*pdwSupportedOptions = 0;
		*pdwEnabledOptions = 0;

		return (E_FAIL);
	}

	return (hRes);
}

//
//	Function:
//		SetInterfaceSafetyOptions (REFIID riid, DWORD dwSupportedOptions,
//			DWORD dwEnabledOptions)
//
//	Parameters:
//		riid				Specifies the interface ID for a given object.
//		dwSupportedOptions	Specifies the options supported for the interface identified by riid.
//		dwEnabledOptions	Specifies the options currently enabled for the interface identified
//							by riid. 
//
//	Return Values:
//		S_OK			Success,  
//		E_NOINTERFACE	riid specifies an interface that is unknown to the object.  
//		E_FAIL			dwOptionSetMask specifies an option that is not supported by the object.  
//
//	Comments:
//		This method takes an interface (either IDispatch for scripting or IPersist* for
//		initialization), a bitmask of options to change (dwOptionSetMask), and the new settings for
//		those options (dwEnabledOptions). The currently supported capabilities are, as described
//		above, INTERFACESAFE_FOR_UNTRUSTED_CALLER and INTERFACESAFE_FOR_UNTRUSTED_DATA
//

STDMETHODIMP
CListBoxControlX::SetInterfaceSafetyOptions (REFIID riid, DWORD dwOptionSetMask,
		DWORD dwEnabledOptions)
{
	HRESULT hRes = E_FAIL;
	LPUNKNOWN pUnknown;

	if (!dwOptionSetMask || !dwEnabledOptions) {
		ATLTRACE (_T ("Null arguments!\n"));
		return (hRes);
	}

	// Get the interface pointer and check if its supported or not.

	hRes = _InternalQueryInterface (riid, (void**) &pUnknown);
	if (FAILED (hRes)) {
		ATLTRACE (_T ("Interface not supported!\n"));
		return (hRes);
	}

	// Release the interface pointer.

	pUnknown->Release ();
	pUnknown = NULL;

	// Store the current safety level.

	m_dwCurrentSafety |= dwEnabledOptions & dwOptionSetMask;

	// check what kind of iterface to the passed iid belongs to.

	if ((riid == IID_IDispatch) && (m_dwCurrentSafety & INTERFACESAFE_FOR_UNTRUSTED_CALLER)) {
		// Client site is making this control safe for scripting. Control is safe for scripting
		// by default, therefore we return success flag.
	}
	else if (((riid == IID_IPersistStreamInit) || (riid == IID_IPersistStorage) ||
		(riid == IID_IPersistPropertyBag)) && (m_dwCurrentSafety &
		INTERFACESAFE_FOR_UNTRUSTED_DATA)) {
		// The client site is making this control safe for intitialization from persistent
		// object data. Since our control is safe for these persistence interfaces, we return
		// success flag.

		return (S_OK);
	}
	else {
		// The control doesn't allow any safe initialization or scripting for any other type of
		// interface.

		return (E_FAIL);
	}

	return (hRes);
}

// IOleControl interface implementation

//
//	Function:
//		OnAmbientPropertyChange (DISPID dispid)
//
//	Parameters:
//		dispID		Dispatch identifier of the ambient property that changed. If the dispid
//					parameter is DISPID_UNKNOWN, it indicates that multiple properties changed.
//					In this case, the control should check all the ambient properties of interest
//					to obtain their current values. 	 
//
//	Return Values:
//		S_OK		Returned in all cases
//
//	Comments:
//		Informs a control that one or more of the container's ambient properties (available through
//		the control site's IDispatch) has changed.
//
// NOTE : This interface retruns S_OK in all cases, even if some error has occured.
//

STDMETHODIMP
CListBoxControlX::OnAmbientPropertyChange (DISPID dispid)
{
	// It is upto control to decide which ambient property it wants to respond to.

	// NOTE: In this control only change to background color is being acknowledges. If you want
	//		 to respond to any other anbient property, add the appropriate code at the appropriate
	//		 place in switch statement.

	switch (dispid) {
		case DISPID_AMBIENT_BACKCOLOR:
		case DISPID_AMBIENT_DISPLAYNAME:
		case DISPID_AMBIENT_FONT:
		case DISPID_AMBIENT_FORECOLOR:
		case DISPID_AMBIENT_LOCALEID:
		case DISPID_AMBIENT_MESSAGEREFLECT:
		case DISPID_AMBIENT_SCALEUNITS:
		case DISPID_AMBIENT_TEXTALIGN:
		case DISPID_AMBIENT_USERMODE:
		case DISPID_AMBIENT_UIDEAD:
		case DISPID_AMBIENT_SHOWGRABHANDLES:
		case DISPID_AMBIENT_SHOWHATCHING:
		case DISPID_AMBIENT_DISPLAYASDEFAULT:
		case DISPID_AMBIENT_SUPPORTSMNEMONICS:
		case DISPID_AMBIENT_AUTOCLIP:
		case DISPID_AMBIENT_APPEARANCE:
		case DISPID_AMBIENT_CODEPAGE:
		case DISPID_AMBIENT_PALETTE:
		case DISPID_AMBIENT_CHARSET:
		case DISPID_AMBIENT_TRANSFERPRIORITY:
		case DISPID_AMBIENT_RIGHTTOLEFT:
		case DISPID_AMBIENT_TOPTOBOTTOM:
		case DISPID_UNKNOWN:
			break;
	}

	return (S_OK);
}

//
//	Function:
//		HRESULT IQuickActivate_QuickActivate (QACONTAINER *pQACont, QACONTROL *pQACtrl)
//
//	Parameters:
//		pQaContainer	[in] Pointer to a structure containing information about the container
//		pQACtrl			[out] Pointer to a structure filled in by the control to return
//						information about the control to the container. The container calling
//						this method must reserve memory for this structure. 

//	Returns:
//		S_OK			The quick activation is proceeding and the QACONTROL structure has been
//						completed.
//		E_FAIL			An unexpected error occurred. The quick activation is not completed.
//
//	Remarks:
//		the container passes a pointer to a QACONTAINER structure. The structure contains
//		pointers to interfaces which are needed by the control and the values of some ambient
//		properties that the control may need. Upon return, the control passes a pointer to a
//		QACONTROL structure that contains pointers to its own interfaces that the container
//		requires, and additional status information.
//

HRESULT
CListBoxControlX::IQuickActivate_QuickActivate (QACONTAINER *pQACont, QACONTROL *pQACtrl)
{
	CComPtr<IFont> pCloneFont;

	if (pQACont->pFont != NULL) {
		// Clone the font.

		if (SUCCEEDED (pQACont->pFont->Clone (&pCloneFont))) {
			pCloneFont->QueryInterface (IID_IFontDisp, (void **)&m_pFont);
		}
	}

	HRESULT hRes = CComControlBase::IQuickActivate_QuickActivate (pQACont, pQACtrl);

	return hRes;
}

//
//	Function:
//		HRESULT IPersistStreamInit_Save (LPSTREAM pStm, BOOL fClearDirty, ATL_PROPMAP_ENTRY* pMap)
//
//	PArameters:
//		pStm			Pointer to the stream where the object is to be saved
//		fClearDirty		Specifies whether to clear the dirty flag
//
//	Results:
//		HRESULT value
//
//	Remarks:
//		Saves an object into the specified stream and indicates whether the object should reset
//		its dirty flag.
//

HRESULT
CListBoxControlX::IPersistStreamInit_Save (LPSTREAM pStm, BOOL fClearDirty,
										   ATL_PROPMAP_ENTRY* pMap)
{
	// Make some initial check for valid pointers and all.

	if (NULL == pStm) {
		return E_POINTER;
	}

    // Save the properties described in the PROP_MAP. Since we don't have any custom
	// properties to be saved, therefore we won't have to save anything else.

	HRESULT hRes = S_OK;
	
	hRes = IPersistStreamInitImpl<CListBoxControlX>::IPersistStreamInit_Save (pStm,
		fClearDirty, pMap);

    return hRes;
}

//
//	Function:
//		HRESULT IPersistStreamInit_Load (LPSTREAM pStm, ATL_PROPMAP_ENTRY* pMap)
//
//	PArameters:
//		pStm		Pointer to the stream where the object is to be loaded
//
//	Results:
//		HRESULT value
//
//	Remarks:
//		This method loads an object from its associated stream.
//

HRESULT
CListBoxControlX::IPersistStreamInit_Load (LPSTREAM pStm, ATL_PROPMAP_ENTRY* pMap)
{
    if (NULL == pStm) {
		return E_POINTER;
	}

    // Load the properties described in the PROP_MAP

	HRESULT hRes = S_OK;
	
	hRes = IPersistStreamInitImpl<CListBoxControlX>::IPersistStreamInit_Load (pStm, pMap);
    
	if (FAILED (hRes)) {
		return hRes;
	}

	m_bRequiresSave = FALSE;

    return hRes;
}

//
//	Function:
//		HRESULT IPersistPropertyBag_Save (LPPROPERTYBAG pPropBag, BOOL fClearDirty,
//										BOOL fSaveAllProperties, ATL_PROPMAP_ENTRY* pMap)
//
//	PArameters:
//		pPropBag			Pointer to the caller's property bag
//		fClearDirty			Specifies whether to clear the dirty flag
//		fSaveAllProperties	Specifies whether to save all properties or just those that
//							have changed
//
//	Results:
//		HRESULT value
//
//	Remarks:
//		This method instructs the object to save its properties to the specified property
//		bag, optionally clearing the object's dirty flag. The caller can request that the
//		object save all properties or that the object save only those that are known to
//		have changed.
//

HRESULT
CListBoxControlX::IPersistPropertyBag_Save (LPPROPERTYBAG pPropBag, BOOL fClearDirty,
											BOOL fSaveAllProperties, ATL_PROPMAP_ENTRY* pMap)
{
	if (NULL == pPropBag) {
		return E_POINTER;
	}

    // Save the properties described in the PROP_MAP

	HRESULT hr = IPersistPropertyBagImpl<CListBoxControlX>::IPersistPropertyBag_Save(pPropBag,
		fClearDirty, fSaveAllProperties, pMap);
 
    return hr;
}

//
//	Function:
//		HRESULT IPersistPropertyBag_Load (LPPROPERTYBAG pPropBag, LPERRORLOG pErrorLog,
//											ATL_PROPMAP_ENTRY* pMap)
//
//	PArameters:
//		pPropBag	Pointer to caller's property bag
//		pErrorLog	Pointer to error log
//
//	Results:
//		HRESULT value
//
//	Remarks:
//		This method instructs the object to initialize itself using the properties available
//		in the property bag, notifying the provided error log object when errors occur.
//		All property storage must take place within this method call as the object cannot hold
//		the IPropertyBag pointer.
//

HRESULT
CListBoxControlX::IPersistPropertyBag_Load (LPPROPERTYBAG pPropBag, LPERRORLOG pErrorLog,
											ATL_PROPMAP_ENTRY* pMap)
{
    if (NULL == pPropBag) {
		return E_POINTER;
	}

    // Load the properties described in the PROP_MAP

	HRESULT hr = IPersistPropertyBagImpl<CListBoxControlX>::IPersistPropertyBag_Load(pPropBag,
						pErrorLog, pMap);

 	if (SUCCEEDED (hr)) {
		m_bRequiresSave = FALSE;
	}
 
    return hr;
}

//
//	Function:
//		STDMETHODIMP InterfaceSupportsErrorInfo (REFIID riid)
//
//	Parameters:
//		riid		IID of interface
//
//	Retruns:
//		HRESULT value
//
//	Remarks:
//

STDMETHODIMP
CListBoxControlX::InterfaceSupportsErrorInfo (REFIID riid)
{
	static const IID* arr[] = 
	{
		&IID_IListBoxControlX,
	};
	for (int i=0; i<sizeof(arr)/sizeof(arr[0]); i++)
	{
		if (InlineIsEqualGUID(*arr[i], riid))
			return S_OK;
	}
	return S_FALSE;
}

//
//	Function:
//		STDMETHODIMP GetCount (long *plNumItems)
//
//	Parameters:
//		plNumItems		Pointer to number of items value
//
//	Retruns:
//		HRESULT value
//
//	Remarks:
//		Call this function to get number of items in the ListBox control.
//		This value includes the number of dividers drawn.
//

STDMETHODIMP
CListBoxControlX::GetCount (long *plNumItems)
{
	// Do some initial checks.

	if (!::IsWindow(m_hWnd)) {
		return Error (OLESTR ("Invalid or NULL window handler for the ListBox control"),
			IID_IListBoxControlX, E_FAIL);
	}

	if (plNumItems == NULL) {
		return Error (OLESTR ("NULL pointer passed in to return the value"),
			IID_IListBoxControlX, E_POINTER);
	}

	*plNumItems = (int)::SendMessage(m_ctlListBox.m_hWnd, CB_GETCOUNT, 0, 0L);
	if (*plNumItems == LB_ERR) {
		return Error (OLESTR ("Error getting number of items from ListBox control"),
			IID_IListBoxControlX, E_FAIL);
	}

	return S_OK;
}

//
//	Function:
//		STDMETHODIMP AddString (BSTR bstrItem)
//
//	Parameters:
//		bstrItem	String to add to control
//
//	Retruns:
//		HRESULT value
//
//	Remarks:
//		Call this function to add text string to ListBox control.
//

STDMETHODIMP
CListBoxControlX::AddString (BSTR bstrItem)
{
	// Do some initial checks.

	if (!::IsWindow(m_hWnd)) {
		return Error (OLESTR ("Invalid or NULL window handler for the ListBox control"),
			IID_IListBoxControlX, E_FAIL);
	}

	long count = 0;
	HRESULT hRes = S_OK;

	hRes = GetCount (&count);
	if (FAILED (hRes)) {
		return hRes;
	}

	// If the count number is one of the indcies, then add a blank line and then add the text
	// string.

	USES_CONVERSION;

	if (FindDividerIndex (count) != -1) {
		CComBSTR bstrItem = _T ("---");
		if (::SendMessage (m_ctlListBox.m_hWnd, LB_ADDSTRING, 0,
			reinterpret_cast<LPARAM>(W2CT (bstrItem))) == LB_ERR) {
			return Error (OLESTR ("Failed to add divider line string in ListBox control"),
				IID_IListBoxControlX, E_FAIL);
		}
	}

	
	if (::SendMessage (m_ctlListBox.m_hWnd, LB_ADDSTRING, 0, reinterpret_cast<LPARAM>(W2CT (bstrItem)))
		== LB_ERR) {
		return Error (OLESTR ("Failed to add text string in ListBox control"),
				IID_IListBoxControlX, E_FAIL);
	}

	return S_OK;
}

//
//	Function:
//		STDMETHODIMP InsertString (long lIndex, BSTR bstrItem, long *plRetPos)
//
//	Parameters:
//		lIndex		Position to insert string
//		bstrItem	String to insert
//		plRetPos	Index where string was inserted
//
//	Retruns:
//		HRESULT value
//
//	Remarks:
//		Call this function to insert text string at the specified zero based index in
//		the ListBox control. If the function inserts the string successfully, then the
//		index where the string got inserted is retruned back in plRetPos argument.
//

STDMETHODIMP
CListBoxControlX::InsertString (long lIndex, BSTR bstrItem, long *plRetPos)
{
	// Do some initial checks.

	if (!::IsWindow(m_hWnd)) {
		return Error (OLESTR ("Invalid or NULL window handler for the ListBox control"),
			IID_IListBoxControlX, E_FAIL);
	}

	if (plRetPos == NULL) {
		return Error (OLESTR ("NULL pointer passed in to return the value"),
			IID_IListBoxControlX, E_POINTER);
	}

	USES_CONVERSION;
	*plRetPos = ::SendMessage (m_ctlListBox.m_hWnd, LB_INSERTSTRING, (int)lIndex,
		reinterpret_cast<LPARAM>(W2CT (bstrItem)));

	if (*plRetPos == LB_ERR || *plRetPos == LB_ERRSPACE) {
		*plRetPos = -1;
		return Error (OLESTR ("Failed to insert text string in ListBox control"),
				IID_IListBoxControlX, E_FAIL);
	}

	return S_OK;
}

//
//	Function:
//		STDMETHODIMP DeleteString (long lIndex, long *plNumItems)
//
//	Parameters:
//		lIndex		Zero based index to delete the string
//		plNumItems	Number of strings in list
//
//	Retruns:
//		HRESULT value
//
//	Remarks:
//		Before deleting the string at given index, it is made sure that user is not
//		trying to delete an item from index corresponding to one of the divider indices.
//		If you want to remove a divider line, then use RemoveDividerIndex method.
//

STDMETHODIMP
CListBoxControlX::DeleteString (long lIndex, long *plNumItems)
{
	// Do some initial checks.

	if (!::IsWindow(m_hWnd)) {
		return Error (OLESTR ("Invalid or NULL window handler for the ListBox control"),
			IID_IListBoxControlX, E_FAIL);
	}

	if (plNumItems == NULL) {
		return Error (OLESTR ("NULL pointer passed in to return the value"),
			IID_IListBoxControlX, E_POINTER);
	}

	// Check if the zero based index corresponds to one of the divider index values. If
	// so, don't let user delete it and return error.

	if (FindDividerIndex (lIndex) != -1) {
		return Error (OLESTR ("Attempt to delete a string at Divider index value"),
			IID_IListBoxControlX, E_FAIL);
	}

	if (::SendMessage (m_ctlListBox.m_hWnd, LB_DELETESTRING, (int)lIndex, 0L) == LB_ERR) {
		*plNumItems = -1;
		return Error (OLESTR ("Failed to insert text string in ListBox control"),
				IID_IListBoxControlX, E_FAIL);
	}

	if (FAILED (GetCount (plNumItems))) {
		return E_FAIL;
	}

	return S_OK;
}

//
//	Function:
//		STDMETHODIMP ResetContent()
//
//	Parameters:
//
//	Retruns:
//		HRESULT value
//
//	Remarks:
//		Call this function to reset all the contents of the ListBox control.
//

STDMETHODIMP
CListBoxControlX::ResetContent()
{
	// Do some initial checks.

	if (!::IsWindow(m_hWnd)) {
		return Error (OLESTR ("Invalid or NULL window handler for the ListBox control"),
			IID_IListBoxControlX, E_FAIL);
	}

	::SendMessage(m_ctlListBox.m_hWnd, LB_RESETCONTENT, 0, 0L);

	return S_OK;
}

//
//	Function:
//		STDMETHODIMP FindString (long lIndexAfter, BSTR bstrItem, long * plRetIndex)
//
//	Parameters:
//		lIndexAfter		Contains the zero-based index of the item before the first
//						item to be searched
//		bstrItem		String to search for
//		plRetIndex		Index at which string is found
//
//	Retruns:
//		HRESULT value
//
//	Remarks:
//		Call this function to find the first string in a list box that contains
//		the specified prefix, without changing the list-box selection. When the search
//		reaches the bottom of the list box, it continues from the top of the list box
//		back to the item specified by lIndexAfter. If lIndexAfter is -1, the entire list
//		box is searched from the beginning.


STDMETHODIMP
CListBoxControlX::FindString (long lIndexAfter, BSTR bstrItem, long *plRetIndex)
{
	// Do some initial checks.

	if (!::IsWindow(m_hWnd)) {
		return Error (OLESTR ("Invalid or NULL window handler for the ListBox control"),
			IID_IListBoxControlX, E_FAIL);
	}

	USES_CONVERSION;
	*plRetIndex =  (long)::SendMessage(m_ctlListBox.m_hWnd, LB_SELECTSTRING,
		(int)lIndexAfter, reinterpret_cast<LPARAM>(W2CT (bstrItem)));

	if (*plRetIndex == LB_ERR) {
		return Error (OLESTR ("Failed to find the text string in ListBox control"),
				IID_IListBoxControlX, E_FAIL);
	}

	return S_OK;
}

//
//	Function:
//		GetCurSel (long *plIndex)
//
//	Parameters:
//		plIndex		Index of selected string
//
//	Retruns:
//		HRESULT value.
//
//	Remarks:
//		Call this member function to retrieve the zero-based index of the currently selected
//		item, if any, in the list box. After getting the current selection, function makes
//		appropriate adjustments to account for the divider lines, if any drawn. The returned
//		zero based index is actually based on the number of text strings present in the
//		list box.
//

STDMETHODIMP
CListBoxControlX::GetCurSel (long *plIndex)
{
	long prevIdx = -1;

	// Do some initial checks.

	if (!::IsWindow(m_hWnd)) {
		return Error (OLESTR ("Invalid or NULL window handler for the ListBox control"),
			IID_IListBoxControlX, E_FAIL);
	}

	if (plIndex == NULL) {
		return Error (OLESTR ("NULL pointer passed in to return the value"),
			IID_IListBoxControlX, E_POINTER);
	}

	if ((*plIndex = ::SendMessage (m_ctlListBox.m_hWnd, LB_GETCURSEL, 0, 0L)) == LB_ERR) {
		return Error (OLESTR ("Failed to get current selected item from ListBox control"),
				IID_IListBoxControlX, E_FAIL);
	}

	// If the selection value corresponds to any of the divider indices, then return -1.

	if (FindDividerIndex (*plIndex) != -1) {
		*plIndex = -1;
		return Error (OLESTR ("Selected item is one of the dividerlines"),
				IID_IListBoxControlX, E_FAIL);
	}

	// Find the value of previous divider index value corresponding to the selection value.

	if ((prevIdx = FindPrevDividerIndex (*plIndex)) != -1) {
		// Compute the actual selection value by offsetting the selection value.

		*plIndex -= (FindDividerIndex (prevIdx) + 1);
	}

	return S_OK;
}

//
//	Function:
//		STDMETHODIMP SetCurSel (long lIndex)
//
//	Parameters:
//		lIndex		Zero based index
//
//	Retruns:
//		HRESULT value
//
//	Remarks:
//		Call this member function to select a string and scroll it into view, if
//		necessary. When the new string is selected, the list box removes the highlight
//		from the previously selected string. 
//

STDMETHODIMP
CListBoxControlX::SetCurSel (long lIndex)
{
	// Do some initial checks.

	if (!::IsWindow(m_hWnd)) {
		return Error (OLESTR ("Invalid or NULL window handler for the ListBox control"),
			IID_IListBoxControlX, E_FAIL);
	}

	if (::SendMessage (m_ctlListBox.m_hWnd, LB_SETCURSEL, (int)lIndex, 0L) == LB_ERR) {
		return Error (OLESTR ("Failed to set the current selection to requested index"),
				IID_IListBoxControlX, E_FAIL);
	}

	return S_OK;
}

//
//	Function:
//		STDMETHODIMP RemoveDividerIndex (long lIndex, long *plRetPos)
//
//	Parameters:
//		lIndex		Zero based value of divider index
//		plRetPos	Pointer to erased index value
//
//	Retruns:
//		HRESULT value
//
//	Remarks:
//		Call this method to remove one of the divider line index vlaue in the list box.
//

STDMETHODIMP
CListBoxControlX::RemoveDividerIndex (long lIndex, long *plRetPos)
{
	if (plRetPos == NULL) {
		return Error (OLESTR ("NULL pointer passed in to return the value"),
			IID_IListBoxControlX, E_POINTER);
	}

	long pos;

	if ((pos = FindDividerIndex (lIndex)) != -1) {
		m_DividerIdxVect.erase (m_DividerIdxVect.begin () + pos);
		*plRetPos = pos;
		return S_OK;
	}

	return Error (OLESTR ("Failed to remove the requested divider index"),
				IID_IListBoxControlX, E_FAIL);
}

//
//	Function:
//		STDMETHODIMP AddDividerIndex (long lIndex)
//
//	Parameters:
//		lIndex		Zero based index value
//
//	Retruns:
//		HRESULT value
//
//	Remarks:
//		Call this method to add a new divider index value. Function makes sure that
//		this index doesn't already existsin the list.
//

STDMETHODIMP
CListBoxControlX::AddDividerIndex (long lIndex)
{
	if (lIndex <= 0) {
		return Error (OLESTR ("Divider index out of range"), IID_IListBoxControlX, E_FAIL);
	}


	// Mkae sure that this index does not exist in the list.

	if (NumDividers () == 0 || FindDividerIndex (lIndex) == -1) {
		m_DividerIdxVect.insert (m_DividerIdxVect.end (), lIndex);
	}

	return S_OK;
}

//
//	Function:
//		long NumDividers () const
//
//	Parameters:
//
//	Retruns:
//		Number of dividers in the list box.
//
//	Remarks:
//		Call this function to get number of dividers present inthe list box.
//

long
CListBoxControlX::NumDividers () const
{
	return m_DividerIdxVect.size ();
}

//
//	Function:
//		long GetDividerAtPos (long lIndex) const
//
//	Parameters:
//		lIndex		Position to find divider index
//
//	Retruns:
//		Divider index value
//
//	Remarks:
//		Call this function to get the zero based index value of the divider in the
//		list box.
//

long
CListBoxControlX::GetDividerAtPos (long lIndex) const
{
	_ASSERTE (lIndex < NumDividers ());
	return m_DividerIdxVect[lIndex];
}

//
//	Function:
//		void RemoveAllIndices ()
//
//	Parameters:
//
//	Retruns:
//
//	Remarks:
//		Call this function to remove all the divider index values from the list.
//

void
CListBoxControlX::RemoveAllIndices ()
{
	m_DividerIdxVect.erase(m_DividerIdxVect.begin(), m_DividerIdxVect.end());
}

//
//	Function:
//		long FindDividerIndex (long lIndex)
//
//	Parameters:
//		lIndex		Zero based divider index
//
//	Retruns:
//		Position in the vector
//
//	Remarks:
//		Call this function to find if a given divider index value is present in the
//		vector ro not. If there is one, then its position in vector is returned. If
//		there is none, then -1 is returned.
//

long
CListBoxControlX::FindDividerIndex (long lIndex)
{
	long retVal = 0;
	// Iterator is used to loop through the vector.
	LONGVECTOR::iterator vectIterator;
	for (vectIterator = m_DividerIdxVect.begin(); vectIterator != m_DividerIdxVect.end();
		vectIterator++) {
			if (*vectIterator == lIndex) {
				return (retVal);
			}

			retVal++;
	}
	
	return (-1);
}

//
//	Function:
//		long FindPrevDividerIndex (long lSel) const
//
//	Parameters:
//
//	Retruns:
//
//	Remarks:
//

long
CListBoxControlX::FindPrevDividerIndex (long lSel) const
{
	int loopIdx = 0;
	long size, idx;
	int prevIdx = -1;
	bool done = false;

	if ((size = NumDividers ()) == 0) {
		return (-1);
	}

	while (!done) {
		idx = GetDividerAtPos (loopIdx);
		if (lSel < idx) {
			return (prevIdx);
		}

		prevIdx = idx;

		if (loopIdx < size - 1) {
			loopIdx++;
		}
		else {
			done = true;
		}
	}

	return (prevIdx);
}

